/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */

package io.opentelemetry.instrumentation.javaagent.runtimemetrics;

import com.google.auto.service.AutoService;
import io.opentelemetry.instrumentation.api.config.Config;
import io.opentelemetry.instrumentation.runtimemetrics.GarbageCollector;
import io.opentelemetry.instrumentation.runtimemetrics.MemoryPools;
import io.opentelemetry.javaagent.spi.ComponentInstaller;
import java.util.Collections;

/** {@link ComponentInstaller} to enable runtime metrics during agent startup. */
@AutoService(ComponentInstaller.class)
public class RuntimeMetricsInstaller implements ComponentInstaller {
  @Override
  public void afterByteBuddyAgent(Config config) {
    if (config.isInstrumentationEnabled(Collections.singleton("runtime-metrics"), true)) {
      GarbageCollector.registerObservers();
      MemoryPools.registerObservers();
    }
  }
}
